Index: src/wp-admin/includes/ms.php
===================================================================
--- src/wp-admin/includes/ms.php	(revision 41199)
+++ src/wp-admin/includes/ms.php	(working copy)
@@ -127,6 +127,11 @@
 			$wpdb->query( "DROP TABLE IF EXISTS `$table`" );
 		}
 
+		$blog_meta_ids = $wpdb->get_col( $wpdb->prepare( "SELECT meta_id FROM $wpdb->blogmeta WHERE blog_id = %d ", $blog_id ) );
+		foreach ( $blog_meta_ids as $mid ) {
+			delete_metadata_by_mid( 'blog', $mid );
+		}
+
 		$wpdb->delete( $wpdb->blogs, array( 'blog_id' => $blog_id ) );
 
 		/**
Index: src/wp-admin/includes/schema.php
===================================================================
--- src/wp-admin/includes/schema.php	(revision 41199)
+++ src/wp-admin/includes/schema.php	(working copy)
@@ -265,6 +265,15 @@
   PRIMARY KEY  (blog_id),
   KEY db_version (db_version)
 ) $charset_collate;
+CREATE TABLE $wpdb->blogmeta (
+  meta_id bigint(20) NOT NULL auto_increment,
+  blog_id bigint(20) NOT NULL default '0',
+  meta_key varchar(255) default NULL,
+  meta_value longtext,
+  PRIMARY KEY  (meta_id),
+  KEY meta_key (meta_key($max_index_length)),
+  KEY blog_id (blog_id)
+) $charset_collate;
 CREATE TABLE $wpdb->registration_log (
   ID bigint(20) NOT NULL auto_increment,
   email varchar(255) NOT NULL default '',
Index: src/wp-admin/includes/upgrade.php
===================================================================
--- src/wp-admin/includes/upgrade.php	(revision 41199)
+++ src/wp-admin/includes/upgrade.php	(working copy)
@@ -1881,6 +1881,11 @@
 			}
 		}
 	}
+
+    // 4.9
+    if ( $wp_current_db_version < 40001 ) {
+        update_site_option( 'site_meta_supported', '1');
+    }
 }
 
 //
Index: src/wp-includes/load.php
===================================================================
--- src/wp-includes/load.php	(revision 41199)
+++ src/wp-includes/load.php	(working copy)
@@ -517,7 +517,7 @@
 	}
 
 	if ( function_exists( 'wp_cache_add_global_groups' ) ) {
-		wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'site-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites' ) );
+		wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'site-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'blog_meta' ) );
 		wp_cache_add_non_persistent_groups( array( 'counts', 'plugins' ) );
 	}
 }
Index: src/wp-includes/ms-blogs.php
===================================================================
--- src/wp-includes/ms-blogs.php	(revision 41199)
+++ src/wp-includes/ms-blogs.php	(working copy)
@@ -465,6 +465,7 @@
 	wp_cache_delete( $domain_path_key, 'blog-id-cache' );
 	wp_cache_delete( 'current_blog_' . $blog->domain, 'site-options' );
 	wp_cache_delete( 'current_blog_' . $blog->domain . $blog->path, 'site-options' );
+	wp_cache_delete( $blog_id, 'blog_meta' );
 
 	/**
 	 * Fires immediately after a site has been removed from the object cache.
@@ -541,21 +542,25 @@
  * Adds any sites from the given ids to the cache that do not already exist in cache.
  *
  * @since 4.6.0
+ * @since 4.9.0 The $update_meta_cache parameter was added.
  * @access private
  *
  * @see update_site_cache()
+ * @see update_sitemeta_cache()
+ *
  * @global wpdb $wpdb WordPress database abstraction object.
  *
- * @param array $ids ID list.
+ * @param array $ids               ID list.
+ * @param bool  $update_meta_cache Optional. Whether to update the meta cache. Default true.
  */
-function _prime_site_caches( $ids ) {
+function _prime_site_caches( $ids, $update_meta_cache = true ) {
 	global $wpdb;
 
 	$non_cached_ids = _get_non_cached_ids( $ids, 'sites' );
 	if ( ! empty( $non_cached_ids ) ) {
 		$fresh_sites = $wpdb->get_results( sprintf( "SELECT * FROM $wpdb->blogs WHERE blog_id IN (%s)", join( ",", array_map( 'intval', $non_cached_ids ) ) ) );
 
-		update_site_cache( $fresh_sites );
+		update_site_cache( $fresh_sites, $update_meta_cache );
 	}
 }
 
@@ -566,15 +571,35 @@
  *
  * @param array $sites Array of site objects.
  */
-function update_site_cache( $sites ) {
+function update_site_cache( $sites, $update_meta_cache = true ) {
 	if ( ! $sites ) {
 		return;
 	}
-
+    $site_ids = array();
 	foreach ( $sites as $site ) {
-		wp_cache_add( $site->blog_id, $site, 'sites' );
+        $site_ids[] = $site->blog_id;
+	    wp_cache_add( $site->blog_id, $site, 'sites' );
 		wp_cache_add( $site->blog_id . 'short', $site, 'blog-details' );
 	}
+
+    if ( $update_meta_cache ) {
+        update_sitemeta_cache( $site_ids );
+    }
+}
+
+/**
+ * Updates metadata cache for list of site IDs.
+ *
+ * Performs SQL query to retrieve all metadata for the sites matching `$site_ids` and stores them in the cache.
+ * Subsequent calls to `get_site_meta()` will not need to query the database.
+ *
+ * @since 4.9.0
+ *
+ * @param array $site_ids List of site IDs.
+ * @return array|false Returns false if there is nothing to update. Returns an array of metadata on success.
+ */
+function update_sitemeta_cache( $site_ids ) {
+	return update_meta_cache( 'blog', $site_ids );
 }
 
 /**
@@ -638,6 +663,7 @@
 	return $query->query( $args );
 }
 
+
 /**
  * Retrieve option value for a given blog id based on name of option.
  *
@@ -842,7 +868,7 @@
 			if ( is_array( $global_groups ) ) {
 				wp_cache_add_global_groups( $global_groups );
 			} else {
-				wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'site-details' ) );
+				wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'site-details', 'blog_meta' ) );
 			}
 			wp_cache_add_non_persistent_groups( array( 'counts', 'plugins' ) );
 		}
@@ -916,7 +942,7 @@
 			if ( is_array( $global_groups ) ) {
 				wp_cache_add_global_groups( $global_groups );
 			} else {
-				wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'site-details' ) );
+				wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'site-details', 'blog_meta' ) );
 			}
 			wp_cache_add_non_persistent_groups( array( 'counts', 'plugins' ) );
 		}
Index: src/wp-includes/option.php
===================================================================
--- src/wp-includes/option.php	(revision 41199)
+++ src/wp-includes/option.php	(working copy)
@@ -225,7 +225,7 @@
 	if ( empty($site_id) )
 		$site_id = $wpdb->siteid;
 
-	$core_options = array('site_name', 'siteurl', 'active_sitewide_plugins', '_site_transient_timeout_theme_roots', '_site_transient_theme_roots', 'site_admins', 'can_compress_scripts', 'global_terms_enabled', 'ms_files_rewriting' );
+	$core_options = array('site_name', 'siteurl', 'active_sitewide_plugins', '_site_transient_timeout_theme_roots', '_site_transient_theme_roots', 'site_admins', 'can_compress_scripts', 'global_terms_enabled', 'ms_files_rewriting', 'site_meta_supported' );
 
 	$core_options_in = "'" . implode("', '", $core_options) . "'";
 	$options = $wpdb->get_results( $wpdb->prepare("SELECT meta_key, meta_value FROM $wpdb->sitemeta WHERE meta_key IN ($core_options_in) AND site_id = %d", $site_id) );
@@ -2059,3 +2059,151 @@
 
 	return $registered[ $option ]['default'];
 }
+
+
+/**
+ * Add meta data field to a site.
+ *
+ * Site meta data is called "Custom Fields" on the Administration Screen.
+ *
+ * @since 4.9.0
+ *
+ * @param int    $site_id    Site ID.
+ * @param string $meta_key   Metadata name.
+ * @param mixed  $meta_value Metadata value. Must be serializable if non-scalar.
+ * @param bool   $unique     Optional. Whether the same key should not be added.
+ *                           Default false.
+ * @return int|false Meta ID on success, false on failure.
+ */
+function add_site_meta( $site_id, $meta_key, $meta_value, $unique = false ) {
+    if( !is_multisite() ){
+        return false;
+    }
+    // If site meta not installed, quit early
+    if( !get_site_option('site_meta_supported') ){
+        return false;
+    }
+
+    $added = add_metadata( 'blog', $site_id, $meta_key, $meta_value, $unique );
+
+    // Bust site query cache.
+    if ( $added ) {
+        wp_cache_set( 'last_changed', microtime(), 'sites' );
+    }
+
+    return $added;
+}
+
+/**
+ * Remove metadata matching criteria from a site.
+ *
+ * You can match based on the key, or key and value. Removing based on key and
+ * value, will keep from removing duplicate metadata with the same key. It also
+ * allows removing all metadata matching key, if needed.
+ *
+ * @since 4.9.0
+ *
+ * @param int    $site_id    Site ID.
+ * @param string $meta_key   Metadata name.
+ * @param mixed  $meta_value Optional. Metadata value. Must be serializable if
+ *                           non-scalar. Default empty.
+ * @return bool True on success, false on failure.
+ */
+function delete_site_meta( $site_id, $meta_key, $meta_value = '' ) {
+    if( !is_multisite() ){
+        return false;
+    }
+    // If site meta not installed, quit early
+    if( !get_site_option('site_meta_supported') ){
+        return false;
+    }
+
+    $deleted = delete_metadata( 'blog', $site_id, $meta_key, $meta_value );
+
+    // Bust site query cache.
+    if ( $deleted ) {
+        wp_cache_set( 'last_changed', microtime(), 'sites' );
+    }
+
+    return $deleted;
+}
+
+/**
+ * Retrieve site meta field for a site.
+ *
+ * @since 4.9.0
+ *
+ * @param int    $site_id Site ID.
+ * @param string $key     Optional. The meta key to retrieve. By default, returns
+ *                        data for all keys. Default empty.
+ * @param bool   $single  Optional. Whether to return a single value. Default false.
+ * @return mixed Will be an array if $single is false. Will be value of meta data
+ *               field if $single is true.
+ */
+function get_site_meta( $site_id, $key = '', $single = false ) {
+    if( !is_multisite() ){
+        return false;
+    }
+    // If site meta not installed, quit early
+    if( !get_site_option('site_meta_supported') ){
+        return false;
+    }
+    return get_metadata( 'blog', $site_id, $key, $single );
+}
+
+/**
+ * Update site meta field based on site ID.
+ *
+ * Use the $prev_value parameter to differentiate between meta fields with the
+ * same key and site ID.
+ *
+ * If the meta field for the site does not exist, it will be added.
+ *
+ * @since 4.9.0
+ *
+ * @param int    $site_id    Site ID.
+ * @param string $meta_key   Metadata key.
+ * @param mixed  $meta_value Metadata value. Must be serializable if non-scalar.
+ * @param mixed  $prev_value Optional. Previous value to check before removing.
+ *                           Default empty.
+ * @return int|bool Meta ID if the key didn't exist, true on successful update,
+ *                  false on failure.
+ */
+function update_site_meta( $site_id, $meta_key, $meta_value, $prev_value = '' ) {
+    if( !is_multisite() ){
+        return false;
+    }
+    // If site meta not installed, quit early
+    if( !get_site_option('site_meta_supported') ){
+        return false;
+    }
+
+    $updated = update_metadata( 'blog', $site_id, $meta_key, $meta_value, $prev_value );
+
+    // Bust site query cache.
+    if ( $updated ) {
+        wp_cache_set( 'last_changed', microtime(), 'sites' );
+    }
+
+    return $updated;
+}
+
+/**
+ * Delete everything from site meta matching meta key.
+ *
+ * @since 4.9.0
+ *
+ * @param string $meta_key Metadata key to search for when deleting.
+ * @return bool Whether the site meta key was deleted from the database.
+ */
+function delete_site_meta_by_key( $meta_key ) {
+    if( !is_multisite() ){
+        return false;
+    }
+    // If site meta not installed, quit early
+    if( !get_site_option('site_meta_supported') ){
+        return false;
+    }
+
+    return delete_metadata( 'blog', null, $meta_key, '', true );
+}
\ No newline at end of file
Index: src/wp-includes/version.php
===================================================================
--- src/wp-includes/version.php	(revision 41199)
+++ src/wp-includes/version.php	(working copy)
@@ -11,7 +11,7 @@
  *
  * @global int $wp_db_version
  */
-$wp_db_version = 38590;
+$wp_db_version = 40001;
 
 /**
  * Holds the TinyMCE version
Index: src/wp-includes/wp-db.php
===================================================================
--- src/wp-includes/wp-db.php	(revision 41199)
+++ src/wp-includes/wp-db.php	(working copy)
@@ -271,7 +271,7 @@
 	 * @see wpdb::tables()
 	 * @var array
 	 */
-	var $ms_global_tables = array( 'blogs', 'signups', 'site', 'sitemeta',
+	var $ms_global_tables = array( 'blogs', 'blogmeta', 'signups', 'site', 'sitemeta',
 		'sitecategories', 'registration_log', 'blog_versions' );
 
 	/**
@@ -383,6 +383,15 @@
 	public $blogs;
 
 	/**
+	 * Multisite Blog Metadata table
+	 *
+	 * @since 4.9.0
+	 * @access public
+	 * @var string
+	 */
+	public $blogmeta;
+
+	/**
 	 * Multisite Blog Versions table
 	 *
 	 * @since 3.0.0
Index: src/wp-settings.php
===================================================================
--- src/wp-settings.php	(revision 41199)
+++ src/wp-settings.php	(working copy)
@@ -95,6 +95,8 @@
 // Load early WordPress files.
 require( ABSPATH . WPINC . '/compat.php' );
 require( ABSPATH . WPINC . '/class-wp-list-util.php' );
+require( ABSPATH . WPINC . '/formatting.php' );
+require( ABSPATH . WPINC . '/meta.php' );
 require( ABSPATH . WPINC . '/functions.php' );
 require( ABSPATH . WPINC . '/class-wp-matchesmapregex.php' );
 require( ABSPATH . WPINC . '/class-wp.php' );
@@ -142,7 +144,7 @@
 // Load most of WordPress.
 require( ABSPATH . WPINC . '/class-wp-walker.php' );
 require( ABSPATH . WPINC . '/class-wp-ajax-response.php' );
-require( ABSPATH . WPINC . '/formatting.php' );
+
 require( ABSPATH . WPINC . '/capabilities.php' );
 require( ABSPATH . WPINC . '/class-wp-roles.php' );
 require( ABSPATH . WPINC . '/class-wp-role.php' );
@@ -157,7 +159,6 @@
 require( ABSPATH . WPINC . '/class-wp-user-query.php' );
 require( ABSPATH . WPINC . '/class-wp-session-tokens.php' );
 require( ABSPATH . WPINC . '/class-wp-user-meta-session-tokens.php' );
-require( ABSPATH . WPINC . '/meta.php' );
 require( ABSPATH . WPINC . '/class-wp-meta-query.php' );
 require( ABSPATH . WPINC . '/class-wp-metadata-lazyloader.php' );
 require( ABSPATH . WPINC . '/general-template.php' );
Index: tests/phpunit/includes/testcase.php
===================================================================
--- tests/phpunit/includes/testcase.php	(revision 41199)
+++ tests/phpunit/includes/testcase.php	(working copy)
@@ -323,7 +323,7 @@
 			$wp_object_cache->__remoteset();
 		}
 		wp_cache_flush();
-		wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'site-details' ) );
+		wp_cache_add_global_groups( array( 'users', 'userlogins', 'usermeta', 'user_meta', 'useremail', 'userslugs', 'site-transient', 'site-options', 'blog-lookup', 'blog-details', 'rss', 'global-posts', 'blog-id-cache', 'networks', 'sites', 'site-details', 'blog_meta' ) );
 		wp_cache_add_non_persistent_groups( array( 'comment', 'counts', 'plugins' ) );
 	}
 
Index: tests/phpunit/tests/multisite/siteMeta.php
===================================================================
--- tests/phpunit/tests/multisite/siteMeta.php	(revision 0)
+++ tests/phpunit/tests/multisite/siteMeta.php	(working copy)
@@ -0,0 +1,166 @@
+<?php
+
+if ( is_multisite() ) :
+/**
+ * @group ms-site
+ * @group multisite
+ * @group meta
+ * @ticket 37923
+ */
+class Tests_Multisite_Site_Meta extends WP_UnitTestCase {
+	protected static $site_id;
+	protected static $site_id2;
+
+    function setUp() {
+        global $wpdb;
+        parent::setUp();
+        $this->suppress = $wpdb->suppress_errors();
+
+        update_site_option( 'site_meta_supported', 1 );
+    }
+
+    function tearDown() {
+        global $wpdb;
+
+        update_site_option( 'site_meta_supported', 0 );
+        $wpdb->suppress_errors( $this->suppress );
+
+        parent::tearDown();
+    }
+
+	public static function wpSetUpBeforeClass( $factory ) {
+		self::$site_id = $factory->blog->create( array( 'domain' => 'wordpress.org', 'path' => '/' ) );
+		self::$site_id2 = $factory->blog->create( array( 'domain' => 'wordpress.org', 'path' => '/foo/' ) );
+	}
+
+	public static function wpTearDownAfterClass() {
+		wpmu_delete_blog( self::$site_id, true );
+		wpmu_delete_blog( self::$site_id2, true );
+
+		wp_update_network_site_counts();
+        delete_site_option( 'site_meta_supported');
+	}
+
+	public function test_add() {
+		$this->assertNotEmpty( add_site_meta( self::$site_id, 'foo', 'bar' ) );
+	}
+
+	public function test_add_unique() {
+		$this->assertNotEmpty( add_site_meta( self::$site_id, 'foo', 'bar' ) );
+		$this->assertFalse( add_site_meta( self::$site_id, 'foo', 'bar', true ) );
+	}
+
+	public function test_delete() {
+		add_site_meta( self::$site_id, 'foo', 'bar' );
+
+		$this->assertTrue( delete_site_meta( self::$site_id, 'foo' ) );
+	}
+
+	public function test_delete_with_invalid_meta_key_should_return_false() {
+		$this->assertFalse( delete_site_meta( self::$site_id, 'foo' ) );
+	}
+
+	public function test_delete_should_respect_meta_value() {
+		add_site_meta( self::$site_id, 'foo', 'bar' );
+		add_site_meta( self::$site_id, 'foo', 'baz' );
+
+		$this->assertTrue( delete_site_meta( self::$site_id, 'foo', 'bar' ) );
+
+		$metas = get_site_meta( self::$site_id, 'foo', false );
+		$this->assertSame( array( 'baz' ), $metas );
+	}
+
+	public function test_get_with_no_key_should_fetch_all_keys() {
+		add_site_meta( self::$site_id, 'foo', 'bar' );
+		add_site_meta( self::$site_id, 'foo1', 'baz' );
+
+		$found = get_site_meta( self::$site_id );
+		$expected = array(
+			'foo' => array( 'bar' ),
+			'foo1' => array( 'baz' ),
+		);
+
+		$this->assertEqualSets( $expected, $found );
+	}
+
+	public function test_get_with_key_should_fetch_all_for_key() {
+		add_site_meta( self::$site_id, 'foo', 'bar' );
+		add_site_meta( self::$site_id, 'foo', 'baz' );
+		add_site_meta( self::$site_id, 'foo1', 'baz' );
+
+		$found = get_site_meta( self::$site_id, 'foo' );
+		$expected = array( 'bar', 'baz' );
+
+		$this->assertEqualSets( $expected, $found );
+	}
+
+	public function test_get_should_respect_single_true() {
+		add_site_meta( self::$site_id, 'foo', 'bar' );
+		add_site_meta( self::$site_id, 'foo', 'baz' );
+
+		$found = get_site_meta( self::$site_id, 'foo', true );
+		$this->assertEquals( 'bar', $found );
+	}
+
+	public function test_update_should_pass_to_add_when_no_value_exists_for_key() {
+		$actual = update_site_meta( self::$site_id, 'foo', 'bar' );
+		$this->assertInternalType( 'int', $actual );
+		$this->assertNotEmpty( $actual );
+
+		$meta = get_site_meta( self::$site_id, 'foo', true );
+		$this->assertSame( 'bar', $meta );
+	}
+
+	public function test_update_should_return_true_when_updating_existing_value_for_key() {
+		add_site_meta( self::$site_id, 'foo', 'bar' );
+
+		$actual = update_site_meta( self::$site_id, 'foo', 'baz' );
+		$this->assertTrue( $actual );
+
+		$meta = get_site_meta( self::$site_id, 'foo', true );
+		$this->assertSame( 'baz', $meta );
+	}
+
+	public function test_delete_by_key() {
+		add_site_meta( self::$site_id, 'unique_delete_by_key', 'value', true );
+		add_site_meta( self::$site_id2, 'unique_delete_by_key', 'value', true );
+
+		$this->assertEquals( 'value', get_site_meta( self::$site_id, 'unique_delete_by_key', true ) );
+		$this->assertEquals( 'value', get_site_meta( self::$site_id2, 'unique_delete_by_key', true ) );
+
+		$this->assertTrue( delete_site_meta_by_key( 'unique_delete_by_key' ) );
+
+		$this->assertEquals( '', get_site_meta( self::$site_id, 'unique_delete_by_key', true ) );
+		$this->assertEquals( '', get_site_meta( self::$site_id2, 'unique_delete_by_key', true ) );
+	}
+
+	public function test_site_meta_should_be_deleted_when_site_is_deleted() {
+		$site_id = self::factory()->blog->create( array( 'domain' => 'foo.org', 'path' => '/' ) );
+
+		add_site_meta( $site_id, 'foo', 'bar' );
+		add_site_meta( $site_id, 'foo1', 'bar' );
+
+		$this->assertSame( 'bar', get_site_meta( $site_id, 'foo', true ) );
+		$this->assertSame( 'bar', get_site_meta( $site_id, 'foo1', true ) );
+
+		wpmu_delete_blog( $site_id, true );
+
+		$this->assertSame( '', get_site_meta( $site_id, 'foo', true ) );
+		$this->assertSame( '', get_site_meta( $site_id, 'foo1', true ) );
+	}
+
+	public function test_disabled_site_meta_by_filter(){
+        $site_id = self::factory()->blog->create( array( 'domain' => 'foo.org', 'path' => '/' ) );
+        add_filter( 'pre_site_option_site_meta_supported', '__return_null' );
+
+	    $this->assertSame( null, get_site_option('site_meta_supported') );
+	    $this->assertSame( false, add_site_meta( $site_id, 'wibble', 'test1' ) );
+	    $this->assertSame( false, update_site_meta( $site_id, 'wibble', 'test2' ) );
+        $this->assertSame( false, get_site_meta( $site_id, 'wibble' ) );
+	    $this->assertSame( false, delete_site_meta( $site_id, 'wibble' ) );
+
+        remove_filter( 'pre_site_option_site_meta_supported', '__return_null' );
+    }
+}
+
+endif;

Property changes on: tests/phpunit/tests/multisite/siteMeta.php
___________________________________________________________________
Added: svn:executable
## -0,0 +1 ##
+*
\ No newline at end of property

Index: src/wp-includes/class-wp-site-query.php
===================================================================
--- src/wp-includes/class-wp-site-query.php	(revision 41199)
+++ src/wp-includes/class-wp-site-query.php	(working copy)
@@ -39,6 +39,7 @@
 		'limits'  => '',
 	);
 
+
 	/**
 	 * Date query container.
 	 *
@@ -136,7 +137,9 @@
 	 *     @type string       $search            Search term(s) to retrieve matching sites for. Default empty.
 	 *     @type array        $search_columns    Array of column names to be searched. Accepts 'domain' and 'path'.
 	 *                                           Default empty array.
 	 *     @type bool         $update_site_cache Whether to prime the cache for found sites. Default false.
 	 * }
 	 */
 	public function __construct( $query = '' ) {
@@ -172,6 +175,7 @@
 			'count'             => false,
 			'date_query'        => null, // See WP_Date_Query
 			'update_site_cache' => true,
+			'update_site_meta_cache' => true,
 		);
 
 		if ( ! empty( $query ) ) {
@@ -224,19 +228,23 @@
 	 *
 	 * @since 4.6.0
 	 *
+	 * @global wpdb $wpdb WordPress database abstraction object.
+	 *
 	 * @return array|int List of sites, or number of sites when 'count' is passed as a query var.
 	 */
-	public function get_sites() {
-		$this->parse_query();
+    public function get_sites() {
+		global $wpdb;
 
-		/**
-		 * Fires before sites are retrieved.
-		 *
-		 * @since 4.6.0
-		 *
-		 * @param WP_Site_Query &$this Current instance of WP_Site_Query, passed by reference.
-		 */
+       /**
+        * Fires before sites are retrieved.
+		*
+		* @since 4.6.0
+		*
+		* @param WP_Site_Query &$this Current instance of WP_Site_Query, passed by reference.
+		*/
 		do_action_ref_array( 'pre_get_sites', array( &$this ) );
+		$this->parse_query();
+
 
 		// $args can include anything. Only use the args defined in the query_var_defaults to compute the key.
 		$_args = wp_array_slice_assoc( $this->query_vars, array_keys( $this->query_var_defaults ) );
@@ -286,7 +294,7 @@
 
 		// Prime site network caches.
 		if ( $this->query_vars['update_site_cache'] ) {
-			_prime_site_caches( $site_ids );
+            _prime_site_caches( $site_ids, $this->query_vars['update_site_meta_cache'] );
 		}
 
 		// Fetch full site objects from the primed cache.
@@ -365,7 +373,7 @@
 
 			$orderby = implode( ', ', $orderby_array );
 		} else {
-			$orderby = "blog_id $order";
+			$orderby = "$wpdb->blogs.blog_id $order";
 		}
 
 		$number = absint( $this->query_vars['number'] );
@@ -382,23 +390,23 @@
 		if ( $this->query_vars['count'] ) {
 			$fields = 'COUNT(*)';
 		} else {
-			$fields = 'blog_id';
+			$fields = "$wpdb->blogs.blog_id";
 		}
 
 		// Parse site IDs for an IN clause.
 		$site_id = absint( $this->query_vars['ID'] );
 		if ( ! empty( $site_id ) ) {
-			$this->sql_clauses['where']['ID'] = $wpdb->prepare( 'blog_id = %d', $site_id );
+			$this->sql_clauses['where']['ID'] = $wpdb->prepare( "$wpdb->blogs.blog_id = %d", $site_id );
 		}
 
 		// Parse site IDs for an IN clause.
 		if ( ! empty( $this->query_vars['site__in'] ) ) {
-			$this->sql_clauses['where']['site__in'] = "blog_id IN ( " . implode( ',', wp_parse_id_list( $this->query_vars['site__in'] ) ) . ' )';
+			$this->sql_clauses['where']['site__in'] = "$wpdb->blogs.blog_id IN ( " . implode( ',', wp_parse_id_list( $this->query_vars['site__in'] ) ) . ' )';
 		}
 
 		// Parse site IDs for a NOT IN clause.
 		if ( ! empty( $this->query_vars['site__not_in'] ) ) {
-			$this->sql_clauses['where']['site__not_in'] = "blog_id NOT IN ( " . implode( ',', wp_parse_id_list( $this->query_vars['site__not_in'] ) ) . ' )';
+			$this->sql_clauses['where']['site__not_in'] = "$wpdb->blogs.blog_id NOT IN ( " . implode( ',', wp_parse_id_list( $this->query_vars['site__not_in'] ) ) . ' )';
 		}
 
 		$network_id = absint( $this->query_vars['network_id'] );
@@ -670,7 +678,7 @@
 				$parsed = 'CHAR_LENGTH(path)';
 				break;
 			case 'id':
-				$parsed = 'blog_id';
+				$parsed = "$wpdb->blogs.blog_id";
 				break;
 		}
 
@@ -136,7 +137,9 @@
 	 *     @type array        $search_columns    Array of column names to be searched. Accepts 'domain' and 'path'.
 	 *                                           Default empty array.
-	 *     @type bool         $update_site_cache Whether to prime the cache for found sites. Default false.
+	 *     @type bool         $update_site_cache Whether to prime the cache for found sites. Default true.
+     *     @type bool         $update_site_meta_cache Whether to prime the metadata cache for found sites.
+     *                                                   Default true.
 	 * }
 	 */